(ns rlm.ns-rlm
  "A few convienence functions for namespaces."
  (:use [clojure.contrib [ns-utils :only [get-ns]]]))

(defn ns-clear
  "completely removes all aliases, interns, and references from the namespace
     works on *ns* if called with no arguments"
  ([ns]
     (let [ns (the-ns ns)]
       (doall (map (partial ns-unmap ns) (keys (ns-map ns))))
       (doall (map (partial ns-unalias ns) (keys (ns-aliases ns)))) nil))
    ([] (ns-clear *ns*)))

(defn ns-clone-fn
  "the calling namespace (*ns*) gets all the referenced vars as the target namespace
     anything that the target namespace doesn't have, ns won't have.  Anything it does
     have, *ns* will have."
  [target-ns]
  (ns-clear *ns*)
  (doall (map (fn [[ns-alias ns]] (alias ns-alias (.name ns))) (ns-aliases target-ns)))
  (doall (map (fn [[sym var]]
		(cond (var? var)
		      (. *ns* (refer sym var))
		      (class? var) 
		      (. *ns* (importClass var))))
		      ;;(eval `(clojure.core/import*  ~(.getName var)))))
		  (ns-map target-ns))) nil)

(defmacro ns-clone
 ([target-ns-name]
    `(do
       (require '~target-ns-name)
       (ns-clone-fn (get-ns '~target-ns-name)))))

(defn ls
  "lists all the vars defined in the namespace"
  ([ns] (doall (map println (keys (ns-map ns)))) nil)
  ([] (ls *ns*)))


